<?php

namespace Drupal\library_manager\Tests;

/**
 * Tests library auto load.
 *
 * @group library_manager
 */
class LibraryAutoLoadTest extends TestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = ['library_manager', 'library_manager_test', 'node'];

  /**
   * Tests that library loads automatically according to configured conditions.
   */
  public function testAutoLoad() {
    $this->createContentType(['type' => 'page']);
    $this->createNode();
    $this->createNode();

    $edit = [
      'load' => TRUE,
      'visibility[request_path][pages]' => '/node/1',
    ];

    $this->drupalPostForm('admin/structure/library/definition/alpha/edit', $edit, 'Save');

    $this->drupalGet('node/1');
    $this->assertRaw('/sites/default/files/libraries/custom/alpha/example.css');
    $this->assertRaw('/sites/default/files/libraries/custom/alpha/example.js');

    $this->drupalGet('node/2');
    $this->assertNoRaw('/sites/default/files/libraries/custom/alpha/example.css');
    $this->assertNoRaw('/sites/default/files/libraries/custom/alpha/example.js');

    // Negate the condition and check the pages again.
    $edit['visibility[request_path][negate]'] = TRUE;
    $this->drupalPostForm('admin/structure/library/definition/alpha/edit', $edit, 'Save');
    $this->drupalGet('admin/structure/library/definition/alpha/edit');

    drupal_flush_all_caches();

    $this->drupalGet('node/1');
    $this->assertNoRaw('/sites/default/files/libraries/custom/alpha/example.css');
    $this->assertNoRaw('/sites/default/files/libraries/custom/alpha/example.js');

    $this->drupalGet('node/2');
    $this->assertRaw('/sites/default/files/libraries/custom/alpha/example.css');
    $this->assertRaw('/sites/default/files/libraries/custom/alpha/example.js');
  }

}
