<?php

namespace Drupal\library_manager\Tests;

/**
 * Tests the Library definition interface.
 *
 * @group library_manager
 */
class LibraryDefinitionTest extends TestBase {

  /**
   * Tests collection page.
   */
  public function testDefinitionCollectionPage() {
    $this->drupalGet('admin/structure/library/definition');

    $this->assertPageTitle('Library definitions');

    $expected_header = [
      'Machine name',
      'Version',
      'License',
      'Operations',
    ];
    $this->assertTableHeader('//main//table//th', $expected_header);

    $alpha_row = $this->xpath('//td[position() = 1 and text() = "alpha"]/../td');
    $this->assertEqual($alpha_row[1], '1.2.3');
    $this->assertEqual($alpha_row[2], 'GNU-GPL-2.0-or-later');
    $operations = $alpha_row[3]->xpath('.//a');
    $this->assertEqual($operations[0], 'Edit', 'Edit operation was found.');
    $this->assertEqual($operations[1], 'Duplicate', 'Duplicate operation was found.');
    $this->assertEqual($operations[2], 'Delete', 'Delete operation was found');
  }

  /**
   * Tests definition form.
   */
  public function testDefinitionForm() {
    $this->drupalGet('admin/structure/library/definition/add');
    $this->assertPageTitle('Add library definition');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-id")]/label[text() = "Machine name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-mode")]/label[text() = "Register new library"]');
    $this->assertByXpath('//div[contains(@class, "form-item-mode")]/label[text() = "Override existing library"]');
    $this->assertByXpath('//div[contains(@class, "form-item-target")]/label[text() = "Library to override"]');
    $this->assertByXpath('//div[contains(@class, "form-item-remote")]/label[text() = "Remote"]');
    $this->assertByXpath('//div[contains(@class, "form-item-version")]/label[text() = "Version"]');
    $this->assertByXpath('//div[contains(@class, "form-item-license-name")]/label[text() = "Name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-license-url")]/label[text() = "URL"]');
    $this->assertByXpath('//div[contains(@class, "form-item-license-gpl-compatible")]/label[text() = "GPL compatible"]');

    $header = ['Name', 'Size', 'Type', 'Operations'];

    $this->assertByXpath('//fieldset[@id="edit-js"]/legend/span[text() = "JS files"]');
    $this->assertTableHeader('//fieldset[@id="edit-js"]//th', $header);
    $this->assertByXpath('//fieldset[@id="edit-js"]//table//td[text() = "JS files are not configured yet."]');

    $this->assertByXpath('//fieldset[@id="edit-css"]/legend/span[text() = "CSS files"]');
    $this->assertTableHeader('//fieldset[@id="edit-css"]//th', $header);
    $this->assertByXpath('//fieldset[@id="edit-css"]//table//td[text() = "CSS files are not configured yet."]');

    $this->assertByXpath('//fieldset[@id="edit-dependencies"]/legend/span[text() = "Dependencies"]');
    $this->assertByXpath('//fieldset[@id="edit-dependencies"]//input[@type="submit" and @value="Add dependency"]');

    $this->assertByXpath('//div[contains(@class, "form-item-load")]/label[text() = "Load the library automatically according to visibility rules"]');

    $tabs_prefix = '//div[contains(@class, "form-type-vertical-tabs")]';
    $this->assertByXpath($tabs_prefix . '/label[text() = "Visibility"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/summary[text() = "Current Theme"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/div//label[text() = "Theme"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/div//label[text() = "Negate the condition"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/summary[text() = "Request Path"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/div//label[text() = "Pages"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/div//label[text() = "Negate the condition"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/summary[text() = "User Role"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/div//fieldset/legend/span[text() = "When the user has the following roles"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/div//label[text() = "Negate the condition"]');

    $edit = [
      'id' => 'foo',
      'mode' => 'override',
      'target' => 'core/backbone',
      'remote' => 'https://www.drupal.org/foo',
      'version' => '3.4.5',
      'license[name]' => 'MIT',
      'license[url]' => 'http://example.com/MIT-LICENSE.txt',
      'license[gpl-compatible]' => TRUE,
      'library_dependencies[0]' => 'core/jquery',
      'load' => TRUE,
      'visibility[current_theme][theme]' => 'classy',
      'visibility[current_theme][negate]' => TRUE,
      'visibility[request_path][pages]' => '/user',
      'visibility[user_role][roles][anonymous]' => TRUE,
    ];

    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertStatusMessage('Library definition has been created.');

    // Check default form values for newly created definition.
    $this->drupalGet('admin/structure/library/definition/foo/edit');

    $this->assertByXpath('//input[@name="id" and @value="foo"]');
    $this->assertByXpath('//input[@name="mode" and @value="override"]');
    $this->assertByXpath('//input[@name="target" and @value="core/backbone"]');
    $this->assertByXpath('//input[@name="remote" and @value="https://www.drupal.org/foo"]');
    $this->assertByXpath('//input[@name="version" and @value="3.4.5"]');
    $this->assertByXpath('//input[@name="license[name]" and @value="MIT"]');
    $this->assertByXpath('//input[@name="license[url]" and @value="http://example.com/MIT-LICENSE.txt"]');
    $this->assertByXpath('//input[@name="license[gpl-compatible]" and @checked]');
    $this->assertByXpath('//input[@name="library_dependencies[0]" and @value="core/jquery"]');
    $this->assertByXpath('//input[@name="load" and @checked="checked"]');
    $this->assertByXpath('//select[@name="visibility[current_theme][theme]"]/option[@value = "classy" and @selected = "selected"]');
    $this->assertByXpath('//input[@name="visibility[current_theme][negate]" and @checked = "checked"]');
    $this->assertByXpath('//textarea[@name="visibility[request_path][pages]" and text() = "/user"]');
    $this->assertByXpath('//input[@name="visibility[request_path][negate]" and not(@checked)]');
    $this->assertByXpath('//input[@name="visibility[user_role][roles][anonymous]" and @checked = "checked"]');
    $this->assertByXpath('//input[@name="visibility[user_role][negate]" and not(@checked)]');

    // Check delete form.
    $delete_url = $this->getAbsoluteUrl($this->xpath('//a[@id="edit-delete"]')[0]['href']);
    $this->drupalGet($delete_url);
    $this->assertPageTitle(t('Are you sure you want to delete the library definition %definition?', ['%definition' => 'foo']));
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Library definitions');
    $this->drupalGet($delete_url);
    $this->drupalPostForm(NULL, [], 'Delete');
    $this->assertStatusMessage(t('Deleted library definition %definition_name.', ['%definition_name' => 'foo']));
    $this->assertPageTitle('Library definitions');
  }

  /**
   * Tests JS file form.
   */
  public function testJsFileForm() {
    $this->drupalGet('admin/structure/library/definition/alpha/edit');
    $this->clickLink('Add JS file');

    $this->assertPageTitle('Add JS file');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-file-name")]/label[text() = "File name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-preprocess")]/label[text() = "Preprocess"]');
    $this->assertByXpath('//div[contains(@class, "form-item-minified")]/label[text() = "Minified"]');
    $this->assertByXpath('//div[contains(@class, "form-item-external")]/label[text() = "External"]');
    $this->assertByXpath('//div[contains(@class, "form-item-code")]/label[text() = "Code"]');

    // Check file name validation.
    $edit = [
      'file_name' => '/foo.js',
      'preprocess' => FALSE,
      'minified' => TRUE,
      'external' => FALSE,
      'code' => 'console.log(123);',
    ];
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'foo.php';
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'bar/../foo.js';
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'foo.js';
    $this->drupalPostForm(NULL, $edit, 'Save');

    $this->assertStatusMessage('The JS file has been saved.');
    $this->assertPageTitle('Edit library definition');

    $this->clickLinkInRow('foo.js', 'Edit');
    $this->assertPageTitle('Edit JS file');

    // Check default form values for newly created JS file.
    $this->assertByXpath('//input[@name="file_name" and @value="foo.js"]');
    $this->assertByXpath('//input[@name="preprocess" and not(@checked)]');
    $this->assertByXpath('//input[@name="minified" and @checked = "checked"]');
    $this->assertByXpath('//input[@name="external" and not(@checked)]');
    $this->assertByXpath('//textarea[@name="code" and text() = "console.log(123);"]');

    $this->clickLink('Delete');

    $this->assertPageTitle('Are you sure you want to delete the file?');
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Edit library definition');

    $this->clickLinkInRow('foo.js', 'Delete');
    $this->drupalPostForm(NULL, [], 'Delete');

    $this->assertStatusMessage('The JS file has been deleted.');
    $this->assertPageTitle('Edit library definition');
  }

  /**
   * Tests CSS file form.
   */
  public function testCssFileForm() {
    $this->drupalGet('admin/structure/library/definition/alpha/edit');
    $this->clickLink('Add CSS file');

    $this->assertPageTitle('Add CSS file');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-file-name")]/label[text() = "File name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-preprocess")]/label[text() = "Preprocess"]');
    $this->assertByXpath('//div[contains(@class, "form-item-minified")]/label[text() = "Minified"]');
    $this->assertByXpath('//div[contains(@class, "form-item-external")]/label[text() = "External"]');
    $this->assertByXpath('//div[contains(@class, "form-item-url")]/label[text() = "Url"]');
    $this->assertByXpath('//div[contains(@class, "form-item-code")]/label[text() = "Code"]');

    // Check file name validation.
    $edit = [
      'file_name' => '/foo.css',
      'preprocess' => FALSE,
      'minified' => TRUE,
      'weight' => 5,
      'external' => FALSE,
      'url' => 'http://example.com/foo.css',
    ];

    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'foo.php';
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'bar/../foo.css';
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertErrorMessage('The file name is not correct.');

    $edit['file_name'] = 'foo.css';
    $this->drupalPostForm(NULL, $edit, 'Save');

    $this->assertStatusMessage('The CSS file has been saved.');
    $this->assertPageTitle('Edit library definition');

    $this->clickLinkInRow('foo.css', 'Edit');
    $this->assertPageTitle('Edit CSS file');

    // Check default form values for newly created JS file.
    $this->assertByXpath('//input[@name="file_name" and @value="foo.css"]');
    $this->assertByXpath('//input[@name="preprocess" and not(@checked)]');
    $this->assertByXpath('//input[@name="minified" and @checked = "checked"]');
    $this->assertByXpath('//select[@name="weight"]/option[@value=5 and @selected="selected"]');
    $this->assertByXpath('//input[@name="external" and not(@checked)]');
    $this->assertByXpath('//input[@name="url" and @value="http://example.com/foo.css"]');

    $this->clickLink('Delete');

    $this->assertPageTitle('Are you sure you want to delete the file?');
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Edit library definition');

    $this->clickLinkInRow('foo.css', 'Delete');

    $this->drupalPostForm(NULL, [], 'Delete');

    $this->assertStatusMessage('The CSS file has been deleted.');
    $this->assertPageTitle('Edit library definition');
  }

  /**
   * Tests library definition duplication.
   */
  public function testLibraryDefinitionDuplicate() {
    $this->drupalGet('admin/structure/library/definition');
    $this->clickLinkInRow('alpha', 'Duplicate');
    $this->assertPageTitle('Duplicate of alpha');
    $this->assertByXpath('//input[@name="id" and @value="duplicate_of_alpha"]');
    $this->drupalPostForm(NULL, ['id' => 'alpha'], 'Duplicate');
    $this->assertErrorMessage('The machine-readable name is already in use. It must be unique.');
    $this->drupalPostForm(NULL, ['id' => 'beta'], 'Duplicate');
    $this->assertPageTitle('Edit library definition');
    $this->assertByXpath('//input[@name="id" and @value="beta"]');
    $this->assertByXpath('//input[@name="version" and @value="1.2.3"]');
    $this->assertText('example.js');
  }

}
